@@ -8,7 +8,7 @@
 #define GLIBMM_MAJOR_VERSION 2
 
 /* Minor version number of glibmm. */
-#define GLIBMM_MINOR_VERSION 66
+#define GLIBMM_MINOR_VERSION 72
 
 /* Micro version number of glibmm. */
 #define GLIBMM_MICRO_VERSION 0
@@ -22,26 +22,6 @@
 /* Defined when the -Ddebug-refcounting configure argument was given */
 /* #undef GLIBMM_DEBUG_REFCOUNTING */
 
-/* This is always set. This is only for backwards compatibility. */
-#define GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED 1
-
-/* This is always set. This is only for backwards compatibility. */
-#define GLIBMM_EXCEPTIONS_ENABLED 1
-
-/* This is always set. This is only for backwards compatibility. */
-#define GLIBMM_VFUNCS_ENABLED 1
-
-/* This is always set. This is only for backwards compatibility. */
-#define GLIBMM_PROPERTIES_ENABLED 1
-
-/* This is always set. This is only for backwards compatibility.
- * All acceptable C++ compilers have namespace std. */
-#define GLIBMM_HAVE_NAMESPACE_STD 1
-
-/* Dummy macro definition for compatibility with old code that expects
- * it to be defined. */
-#define GLIBMM_USING_STD(Symbol)
-
 #ifdef _WIN32
   /* Win32 compilers have a lot of varation */
 # if defined(_MSC_VER)
@@ -73,20 +53,10 @@
    type. */
 #define GLIBMM_CAN_USE_DYNAMIC_CAST_IN_UNUSED_TEMPLATE_WITHOUT_DEFINITION 1
 
-/* Define if the compiler honors namespaces inside extern "C" blocks. */
-#define GLIBMM_CAN_USE_NAMESPACES_INSIDE_EXTERNC 1
-
-/* Defined when the SUN Forte C++ compiler is being used. */
-/* #undef GLIBMM_COMPILER_SUN_FORTE */
-
 /* Defined if a static member variable may be initialized inline to
    std::string::npos */
 #define GLIBMM_HAVE_ALLOWS_STATIC_INLINE_NPOS 1
 
-/* Defined when time_t is not equivalent to gint32, meaning that it can be
-   used for a method overload */
-#define GLIBMM_HAVE_C_STD_TIME_T_IS_NOT_INT32 1
-
 /* Define if the compiler disambiguates template specializations for const and
    non-const types. */
 #define GLIBMM_HAVE_DISAMBIGUOUS_CONST_TEMPLATE_SPECIALIZATIONS 1
@@ -103,12 +73,6 @@
 /* Define to 1 if wide stream is available. */
 #define GLIBMM_HAVE_WIDE_STREAM 1
 
-/* Define if C++ member functions may refer to member templates. */
-#define GLIBMM_MEMBER_FUNCTIONS_MEMBER_TEMPLATES 1
-
-/* Define if the thread_local keyword is supported. */
-#define GLIBMM_CAN_USE_THREAD_LOCAL 1
-
 #endif /* GLIBMM_CONFIGURE */
 
 #ifdef GLIBMM_MSC
@@ -116,11 +80,8 @@
 # define GLIBMM_HAVE_TEMPLATE_SEQUENCE_CTORS 1
 # define GLIBMM_HAVE_WIDE_STREAM 1
 # define GLIBMM_HAVE_DISAMBIGUOUS_CONST_TEMPLATE_SPECIALIZATIONS 1
-# define GLIBMM_HAVE_C_STD_TIME_T_IS_NOT_INT32 1
 # define GLIBMM_CAN_USE_DYNAMIC_CAST_IN_UNUSED_TEMPLATE_WITHOUT_DEFINITION 1
-# define GLIBMM_CAN_USE_THREAD_LOCAL 1
 # define GLIBMM_CAN_ASSIGN_NON_EXTERN_C_FUNCTIONS_TO_EXTERN_C_CALLBACKS 1
-# define GLIBMM_CAN_USE_NAMESPACES_INSIDE_EXTERNC 1
 # pragma warning (disable: 4786 4355 4800 4181)
 
 /* We have GLIBMM_HAVE_ALLOWS_STATIC_INLINE_NPOS for Visual Studio 2017+ */
@@ -138,11 +99,13 @@
 #ifdef GLIBMM_DLL
 # if defined(GLIBMM_BUILD)
 #  define GLIBMM_API __declspec(dllexport)
-# else
+# elif !defined (__GNUC__)
 #  define GLIBMM_API __declspec(dllimport)
-# endif /* GLIBMM_BUILD - _WINDLL */
+# else /* don't dllimport classes/methods on GCC/MinGW */
+#  define GLIBMM_API
+# endif
+/* Build a static or non-native-Windows library */
 #else
-/* Build a static library or a non-native-Windows library */
 # define GLIBMM_API
 #endif /* GLIBMM_DLL */
 
